function CellMakerX_AllImageToTrain( path_src_images_folder, path_src_images_gt_folder, synIm_size, ...
                                       cell_num_per_img, Lratio, Hratio, ...
                                       num_Train_img, output_path )
%CellMakerX_AllImageToTrain Synthetic Cervical Cell Images Generator
%
%Generate the overlapping cerivcal cell images by
%   1) number of overlapping cells per image
%   2) overlapping ratio between any pair of cells
%
%CellMakerX_AllImageToTrain(path_src_images_folder, path_src_images_gt_folder,...
%                           synIm_size, cell_num_per_img, Lratio, Hratio, ...
%                           num_Train_img, output_path)
%
%Parameters
%----------
%   1. path_src_images_folder: Path of input EDF images.
%   2. path_src_images_gt_folder: Path of input EDF nuclei ground truth images.
%   3. synIm_size: Size of synthetic images, should be 512.
%   4. cell_num_per_img: Number of cells in each image.
%   5. Lratio: Minimum overlap ratio.
%   6. Hratio: Maximum overlap ratio.
%   7. num_Train_img: Number of synthetic images generated.
%   8. output_path: Path of output folder.

%+-----------------------------+----------------------------------+
%| Create on: June 12, 2013    | Last Revision: February 23, 2014 |
%+-----------------------------+----------------------------------+

%==================================
%   Add folders/subfolders under
%   current folder
%==================================
currentfolder = pwd;
addpath(genpath(currentfolder))

%%
%===================================
% Read source images & ground truth
%===================================

    imNum = 0;
    %+--------------------+
    %|    Source image    |
    %+--------------------+
    listing_src_img = dir(path_src_images_folder);
    imNum_src = 0;
    for i = 1:size(listing_src_img,1)
        if listing_src_img(i,1).isdir == 1
            continue;
        end

        imNum_src = imNum_src + 1;
        imSet{imNum_src,1} = imread(strcat(path_src_images_folder,listing_src_img(i,1).name));
    end

    %+-------------------+
    %|   Ground truth    |
    %|(index match imSet)|
    %+-------------------+
    listing_gt_img = dir(path_src_images_gt_folder);
    imNum_gt = 0;
    for i = 1:size(listing_src_img,1)
        if listing_src_img(i,1).isdir == 1
            continue;
        end
        for j = 1:size(listing_gt_img,1)
            if listing_gt_img(j,1).isdir == 1
                continue;
            end
            if strcmp(listing_src_img(i,1).name(1:6), listing_gt_img(j,1).name(1:6))
                imNum_gt = imNum_gt + 1;
                imGTSet{imNum_gt,1} = imread(strcat(path_src_images_gt_folder,listing_gt_img(j,1).name));
            else
                continue;
            end
        end
    end

    %+-------------------------+
    %|Check if src image number|
    %|       equals to         |
    %|   ground truth number   |
    %+-------------------------+
    if imNum_gt == imNum_src
        fprintf('Read source images and ground truth successfully!\n');
        imNum = imNum_gt;
    else
        exit(0);
    end

%%
%===================================
% Extract background from EDF image
%===================================
try
    load(strcat(output_path, 'Variables/background_image_template.mat'), 'BackgroundImg');
    load('ImsSource/src_mat/SceneCytoClumpMaskSet.mat', 'SceneCytoClumpMaskSet');
    
    fprintf('Read Background Data Successfully!\n');
catch
    %+--------------------------------+
    %| Open Scene Segmentation Result |
    %+--------------------------------+
    load('ImsSource/src_mat/SceneCytoClumpMaskSet.mat', 'SceneCytoClumpMaskSet');
   
    %+---------------------------------------------------+
    %| Background Pixels Values List from All EDF Images |
    %+---------------------------------------------------+
    Background_Pixels_List = [];
    for i = 1:imNum
        idx_background = find(SceneCytoClumpMaskSet{i,1} == 0);
        Background_Pixels_List = [Background_Pixels_List; imSet{i,1}(idx_background)];
    end
    
    %+-------------------------------------+
    %|  Randomly select Background Pixels  |
    %|                from                 |
    %|      "Background_Pixels_List"       |
    %+-------------------------------------+
    BackgroundImg = uint8(zeros(synIm_size, synIm_size));
    for i = 1:synIm_size
        for j = 1:synIm_size
            id_bg_pixel_in_list = randperm(size(Background_Pixels_List,1),1);
            BackgroundImg(i,j) = Background_Pixels_List(id_bg_pixel_in_list,1);
        end
    end
    
    %+-------------------------------+
    %|  Smooth the background image  |
    %|              by               |
    %|         Gaussian blur         |
    %+-------------------------------+
    bg_smooth_filter = fspecial('gaussian', [3,3], 0.5);
    BackgroundImg = imfilter(BackgroundImg, bg_smooth_filter, 'replicate');
    
    save(strcat(output_path, 'Variables/background_image_template.mat'), 'BackgroundImg');
    
    fprintf('Generate Background Data Successfully!\n');
end

%%
%==================================================
% Extract free-lying cells from Scene Segmentation
%==================================================
try
    load(strcat(output_path, 'Variables/EDF_AllFreelyingCellPool.mat'), ...
        'Free_lyingCellSet', 'Free_lyingCellMaskSet', 'Free_lyingCellNucleiMaskSet');
    fprintf('Read EDF Free_lying Cells Data Successfully!\n');
catch    
    Free_lyingCellSet = cell(1,1);
    Free_lyingCellMaskSet = cell(1,1);
    Free_lyingCellNucleiMaskSet = cell(1,1);
    
    numFree_lyingCells = 0;
    
    for i = 1:imNum
        sceneMask = SceneCytoClumpMaskSet{i,1};
        sceneStats = regionprops(sceneMask, 'PixelIdxList', 'BoundingBox', 'Image', 'FilledImage');
        nucleusMask = logical(imGTSet{i,1});
        nucleusStats = regionprops(nucleusMask, 'PixelIdxList');
        
        %+-------------------------------------+
        %| Count Nucleus Number for Each Clump |
        %+-------------------------------------+
        for j = 1:size(sceneStats,1)
            idx_scene_clump_j = sceneStats(j,1).PixelIdxList;
            numNucleusInClump = 0;
            Nuclei_Idx_in_Clump = cell(1,1);
            
            for k = 1:size(nucleusStats,1)
                idx_nuclei_k = nucleusStats(k,1).PixelIdxList;
                sharedPixel_NucleiClump = intersect(idx_scene_clump_j, idx_nuclei_k);
                
                %+---------------------------------+
                %| Empty or non-Empty intersection |
                %|              between            |
                %|       Clump j and Nuclei k      |
                %+---------------------------------+                
                if ~isempty(sharedPixel_NucleiClump) == 0
                    continue;
                else
                    if size(sharedPixel_NucleiClump,1) / size(idx_nuclei_k,1) == 1
                        numNucleusInClump = numNucleusInClump + 1;
                        Nuclei_Idx_in_Clump{numNucleusInClump,1} = idx_nuclei_k;
                    else
                        continue;
                    end
                end
            end
            
            %+--------------------------------------------------------+
            %| Put clump with numNucleusInClump == 1 and nuclei info. |
            %|                          into                          |
            %|        Free_lyingCellSet, Free_lyingCellMaskSet        |
            %|                          and                           |
            %|              Free_lyingCellNucleiMaskSet               |
            %+--------------------------------------------------------+
            if numNucleusInClump == 1                
                numFree_lyingCells = numFree_lyingCells + 1;
                
                %+-------------------------+
                %|  Free-lying Cell Image  |
                %|        512 x 512        |
                %+-------------------------+                
                Free_lyingCellSet{numFree_lyingCells,1} = ...
                    imSet{i,1}(round(sceneStats(j,1).BoundingBox(2) ...
                                    :sceneStats(j,1).BoundingBox(2)+sceneStats(j,1).BoundingBox(4)-1),...
                               round(sceneStats(j,1).BoundingBox(1) + 1 ...
                                    :sceneStats(j,1).BoundingBox(1)+sceneStats(j,1).BoundingBox(3)-1));
                
                %+------------------------+
                %|  Free-lying Cell Mask  |
                %|        512 x 512       |
                %+------------------------+
                ClumpMask_EDF = zeros(size(imSet{i,1}));
                ClumpMask_EDF(idx_scene_clump_j) = 1;
                Free_lyingCellMaskSet{numFree_lyingCells,1} = ...
                    ClumpMask_EDF(round(sceneStats(j,1).BoundingBox(2) ...
                                       :sceneStats(j,1).BoundingBox(2)+sceneStats(j,1).BoundingBox(4)-1),...
                                  round(sceneStats(j,1).BoundingBox(1) + 1 ...
                                       :sceneStats(j,1).BoundingBox(1)+sceneStats(j,1).BoundingBox(3)-1));
                
                %+--------------------------------------+
                %| Nuclei Mask for this Free-lying Cell |
                %|              512 x 512               |
                %+--------------------------------------+
                NucleiMask_EDF = zeros(size(imSet{i,1}));
                NucleiMask_EDF(Nuclei_Idx_in_Clump{1,1}) = 1;
                Free_lyingCellNucleiMaskSet{numFree_lyingCells,1} = ...
                    NucleiMask_EDF(round(sceneStats(j,1).BoundingBox(2) ...
                                        :sceneStats(j,1).BoundingBox(2)+sceneStats(j,1).BoundingBox(4)-1),...
                                   round(sceneStats(j,1).BoundingBox(1) + 1 ...
                                        :sceneStats(j,1).BoundingBox(1)+sceneStats(j,1).BoundingBox(3)-1));
            end
        end
    end
    
    %+--------------------+
    %| Check Equalization |
    %+--------------------+
    assert(isequal(size(Free_lyingCellSet), size(Free_lyingCellMaskSet), size(Free_lyingCellNucleiMaskSet)), ...
           'Error! Free-lying cells matrix are NOT equal!')
    
    %+--------------------------------+
    %| Filter the *fake* single cells |
    %|      by prior knowledge        |
    %+--------------------------------+
    sizeMax = 200;
    idx_tobe_removed = [];
    num_cell_removed = 0;
    for i = 1:size(Free_lyingCellSet,1)
        [width, height] = size(Free_lyingCellSet{i,1});
        
        if width > sizeMax || height > sizeMax
            num_cell_removed = num_cell_removed + 1;
            idx_tobe_removed(num_cell_removed,1) = i;
        end
    end

    Free_lyingCellSet(idx_tobe_removed(:)) = [];
    Free_lyingCellMaskSet(idx_tobe_removed(:)) = [];
    Free_lyingCellNucleiMaskSet(idx_tobe_removed(:)) = [];
    
    %+----------------------------+
    %|   Remove Mannual Checked   |
    %|       Single Cells         |
    %+----------------------------+
    Free_lyingCellSet([1, 2, 5, 7, 14, 16, 17, 18, 19, 21]') = [];
    Free_lyingCellMaskSet([1, 2, 5, 7, 14, 16, 17, 18, 19, 21]') = [];
    Free_lyingCellNucleiMaskSet([1, 2, 5, 7, 14, 16, 17, 18, 19, 21]') = [];
    
    save(strcat(output_path, 'Variables/EDF_AllFreelyingCellPool.mat'), ...
        'Free_lyingCellSet', 'Free_lyingCellMaskSet', 'Free_lyingCellNucleiMaskSet');
    
    fprintf('Generate EDF Free_lying Cells Data Successfully!\n');
end
%%
%================================
%       Build Training idx
%      (NO duplicate cell)
%================================
try
    load(strcat(output_path, 'Variables/TrainCellPool.mat'), ...
        'Train_CellSet', 'Train_CellMaskSet', 'Train_CellNucleiMaskSet');
    
    load(strcat(output_path, 'Variables/TTCellPool_metaData.mat'), ...
        'rand_idx_all_img', 'idx_Train', 'NUM_Train_Img');
    
    fprintf('Read Randomly Generated Free-lying Cells for Training Data Set, respectively, Successfully!\n');
catch
    rand_idx_all_img = randperm(size(Free_lyingCellSet,1))';
    NUM_Train_Img = round(1 * size(Free_lyingCellSet,1));

    idx_Train = rand_idx_all_img(1:NUM_Train_Img);

    Train_CellSet = cell(size(idx_Train,1),1);

    Train_CellMaskSet = cell(size(idx_Train,1),1);

    Train_CellNucleiMaskSet = cell(size(idx_Train,1),1);

    for i = 1:size(Train_CellSet,1)
        Train_CellSet{i,1} = Free_lyingCellSet{idx_Train(i,1),1};
        Train_CellMaskSet{i,1} = Free_lyingCellMaskSet{idx_Train(i,1),1};
        Train_CellNucleiMaskSet{i,1} = Free_lyingCellNucleiMaskSet{idx_Train(i,1),1};
    end
    
    save(strcat(output_path, 'Variables/TrainCellPool.mat'), ...
        'Train_CellSet', 'Train_CellMaskSet', 'Train_CellNucleiMaskSet');
    
    save(strcat(output_path, 'Variables/TTCellPool_metaData.mat'), ...
        'rand_idx_all_img', 'idx_Train', 'NUM_Train_Img');
    
    fprintf('Randomly Generate Free-lying Cells for Training Data Set, respectively, Successfully!\n');
end

%%
%==================================
% Homogeneous Space Transformation
%==================================
%+--------------------------------+
%| Required Cell Number is Valid? |
%+--------------------------------+
assert(cell_num_per_img <= size(idx_Train,1), ...
       'Error! Required cell number CANNOT larger than that in the pool!');
   
try
    load(strcat(output_path, 'Variables/TrainTransformedCellPool.mat'), ...
        'Train_Img_Members_Set', 'Train_Img_Members_Mask_Set', 'Train_Img_Members_NucleiMask_Set');
    fprintf('Read Transformed Free-lying Cells (on 512 x 512) for Training Data Set Successfully!\n');
catch
%+------------------------------+
%| Homogeneous for Training set |
%+------------------------------+
Train_Img_Members_Set = cell(num_Train_img,1);
Train_Img_Members_Mask_Set = cell(num_Train_img,1);
Train_Img_Members_NucleiMask_Set = cell(num_Train_img,1);

hit_num_train = zeros(1,1);
cell_num_in_hit = 1;
for i = 1:num_Train_img
    %+---------------------------------------------+
    %| Randomly pick up single cells from the pool |
    %|      by the required number of cells        |
    %|            "cell_num_per_img"               |
    %+---------------------------------------------+
    rand_idx_train_all = randperm(size(Train_CellSet,1));
    idx_pickup_train = rand_idx_train_all(1:cell_num_per_img);
    pickup_Train_CellSet = Train_CellSet(idx_pickup_train(:),1);
    pickup_Train_CellMaskSet = Train_CellMaskSet(idx_pickup_train(:),1);
    pickup_Train_CellNucleiSet = Train_CellNucleiMaskSet(idx_pickup_train(:),1);
   
    for j = 1:size(pickup_Train_CellSet,1)
        theta = rand(1) * 2 * pi;
        delta = 0.4 * rand(1) + 0.8;
        
        %+----------------------------------+
        %| Fix 1st cell in the image center |
        %+----------------------------------+
        if j == 1
            tx = round(synIm_size / 2);
            ty = round(synIm_size / 2);
            
            %+------------------------------+
            %|  Homogeneous Transformation  |
            %+------------------------------+
            [Y_corners,M_homoTrans] = homoSpaceTransform(tx, ty, theta, delta, pickup_Train_CellSet{j,1});

            %+----------------------------------------+
            %| Draw the cell mask on 512 x 512 canvas |
            %+----------------------------------------+
            Canvas = zeros(synIm_size,synIm_size);
            Canvas = roipoly(Canvas, Y_corners(:,1), Y_corners(:,2));
            [y_canvas,x_canvas]=find(Canvas==1);

            Y_tilde = round(inv(M_homoTrans)*[x_canvas';y_canvas';ones(1,size(x_canvas,1))]);     
            idxCellMask = find(pickup_Train_CellMaskSet{j,1} == 1);
            idxYTilde = (sub2ind(size(pickup_Train_CellMaskSet{j,1}), round(Y_tilde(1,:)), round(Y_tilde(2,:))))';

            %+-----------------------------------+
            %| Draw the cell on 512 x 512 canvas |
            %+-----------------------------------+
            tmp_Img = uint8(zeros(synIm_size,synIm_size));
            tmp_pickup_Cell_Mask = false(synIm_size,synIm_size);
            tmp_pickup_Cell_NucleiMask = false(synIm_size,synIm_size);

            for k = 1:size(y_canvas,1)
                idx = idxYTilde(k,1);
                isCellPixel = ismember(idx, idxCellMask);
                if isCellPixel == 1
                    tmp_Img(x_canvas(k,1),y_canvas(k,1)) = pickup_Train_CellSet{j,1}(Y_tilde(1,k), Y_tilde(2,k));
                    tmp_pickup_Cell_Mask(x_canvas(k,1),y_canvas(k,1)) = logical(pickup_Train_CellMaskSet{j,1}(Y_tilde(1,k), Y_tilde(2,k)));
                    tmp_pickup_Cell_NucleiMask(x_canvas(k,1),y_canvas(k,1)) = logical(pickup_Train_CellNucleiSet{j,1}(Y_tilde(1,k), Y_tilde(2,k)));
                end
            end
            
            %+----------------------------------+
            %| Translate tmp_Img* to the center |
            %|           if j == 1              |
            %+----------------------------------+
            img_1stCell_stat = regionprops(tmp_pickup_Cell_Mask, 'Centroid');
            dx = tx - img_1stCell_stat(1,1).Centroid(1,2);
            dy = ty - img_1stCell_stat(1,1).Centroid(1,1);
            
            [xTmpImg, yTmpImg] = find(tmp_pickup_Cell_Mask == 1);
            [xTmpNucleiMask, yTmpNucleiMask] = find(tmp_pickup_Cell_NucleiMask == 1);
            
            Train_Img_Members_Set{i,1}{j,1} = uint8(zeros(synIm_size,synIm_size));
            Train_Img_Members_Mask_Set{i,1}{j,1} = false(synIm_size,synIm_size);
            Train_Img_Members_NucleiMask_Set{i,1}{j,1} = false(synIm_size,synIm_size);

            %+--------------------------+
            %| Move image and cyto mask |
            %+--------------------------+
            for k = 1:size(xTmpImg,1)
                Train_Img_Members_Set{i,1}{j,1}(round(xTmpImg(k,1) + dx), round(yTmpImg(k,1) + dy)) = ...
                    tmp_Img(round(xTmpImg(k,1)), round(yTmpImg(k,1)));
                Train_Img_Members_Mask_Set{i,1}{j,1}(round(xTmpImg(k,1) + dx), round(yTmpImg(k,1) + dy)) = 1;
            end
            
            %+------------------+
            %| Move Nuclei Mask |
            %+------------------+
            for k = 1:size(xTmpNucleiMask,1)
                Train_Img_Members_NucleiMask_Set{i,1}{j,1}(round(xTmpNucleiMask(k,1) + dx), round(yTmpNucleiMask(k,1) + dy)) = 1;
            end
            
            fprintf('Cell %d in synthetic image %d is OK.\n', j,i);
        else
            %+-----------------------------+
            %|     Deposite Rest Cells     |
            %|         by brute way        |
            %+-----------------------------+
            isRatioOK = 0;
            while isRatioOK == 0
                %+------------------------------------------------------+
                %|Generate high hit-rate (tx, ty) by limiting its range |
                %+------------------------------------------------------+
                Cell_1st_stat = regionprops(Train_Img_Members_Mask_Set{i,1}{1,1}, 'Centroid', 'MajorAxisLength', 'MinorAxisLength');
                rangeFactors = [1.1:0.1:1.7]';
                idx_rangeFactors = randi(size(rangeFactors,1));
                rfactor = rangeFactors(idx_rangeFactors,1);
                range_to_use = min(synIm_size, rfactor * Cell_1st_stat(1,1).MajorAxisLength);
                
                
                range_corners(1,1) = max(Cell_1st_stat(1,1).Centroid(1,1) - range_to_use, 0);
                range_corners(1,2) = max(Cell_1st_stat(1,1).Centroid(1,2) - range_to_use, 0);
                
                range_corners(2,1) = max(Cell_1st_stat(1,1).Centroid(1,1) + range_to_use, 0);
                range_corners(2,2) = max(Cell_1st_stat(1,1).Centroid(1,2) - range_to_use, 0);
                
                range_corners(3,1) = max(Cell_1st_stat(1,1).Centroid(1,1) + range_to_use, 0);
                range_corners(3,2) = max(Cell_1st_stat(1,1).Centroid(1,2) + range_to_use, 0);
                
                range_corners(4,1) = max(Cell_1st_stat(1,1).Centroid(1,1) - range_to_use, 0);
                range_corners(4,2) = max(Cell_1st_stat(1,1).Centroid(1,2) + range_to_use, 0);
                
                tmp_range_I = zeros(synIm_size, synIm_size);
                tmp_range_I = roipoly(tmp_range_I, range_corners(:,1), range_corners(:,2));
                
                idx_external_range = find(tmp_range_I == 1);
                idx_1stCell_mask = find(Train_Img_Members_Mask_Set{i,1}{1,1} == 1);
                
                idx_range = setdiff(idx_external_range, idx_1stCell_mask);
                
                rand_idx_range = randi(size(idx_range,1));
                [tx,ty] = ind2sub(size(Train_Img_Members_Mask_Set{i,1}{1,1}), idx_range(rand_idx_range));              
                
                
                [Y_corners,M_homoTrans] = homoSpaceTransform(tx, ty, theta, delta, pickup_Train_CellSet{j,1});
                
                %+----------------------------------------+
                %| Draw the cell mask on 512 x 512 canvas |
                %+----------------------------------------+
                Canvas = zeros(synIm_size,synIm_size);
                Canvas = roipoly(Canvas, Y_corners(:,1), Y_corners(:,2));
                [y_canvas,x_canvas]=find(Canvas==1);

                Y_tilde = round(inv(M_homoTrans)*[x_canvas';y_canvas';ones(1,size(x_canvas,1))]);     
                idxCellMask = find(pickup_Train_CellMaskSet{j,1} == 1);
                idxYTilde = (sub2ind(size(pickup_Train_CellMaskSet{j,1}), round(Y_tilde(1,:)), round(Y_tilde(2,:))))';
                
                %+-----------------------------------+
                %| Draw the cell on 512 x 512 canvas |
                %+-----------------------------------+
                tmp_Img = uint8(zeros(synIm_size,synIm_size));
                tmp_pickup_Cell_Mask = false(synIm_size,synIm_size);
                tmp_pickup_Cell_NucleiMask = false(synIm_size,synIm_size);

                for k = 1:size(y_canvas,1)
                    idx = idxYTilde(k,1);
                    isCellPixel = ismember(idx, idxCellMask);
                    if isCellPixel == 1
                        tmp_Img(x_canvas(k,1),y_canvas(k,1)) = pickup_Train_CellSet{j,1}(Y_tilde(1,k), Y_tilde(2,k));
                        tmp_pickup_Cell_Mask(x_canvas(k,1),y_canvas(k,1)) = logical(pickup_Train_CellMaskSet{j,1}(Y_tilde(1,k), Y_tilde(2,k)));
                        tmp_pickup_Cell_NucleiMask(x_canvas(k,1),y_canvas(k,1)) = logical(pickup_Train_CellNucleiSet{j,1}(Y_tilde(1,k), Y_tilde(2,k)));
                    end
                end
                
                %+-------------------------------+
                %| Check Validity of Ratio Range |
                %+-------------------------------+
                ratioList = zeros(size(Train_Img_Members_Set{i,1},1),1);
                idx_mask_thisCell = find(tmp_pickup_Cell_Mask == 1);
                
                for k = 1:size(Train_Img_Members_Set{i,1},1)
                    idx_mask_prev = find(Train_Img_Members_Mask_Set{i,1}{k,1} == 1);
                    ratioList(k,1) = max(size(intersect(idx_mask_prev, idx_mask_thisCell),1) / size(idx_mask_thisCell,1), ...
                                        size(intersect(idx_mask_prev, idx_mask_thisCell),1) / size(idx_mask_prev,1));                    
                end
                
                if max(ratioList(:,1)) <= Hratio && max(ratioList(:,1)) >= Lratio
                    Train_Img_Members_Set{i,1}{j,1} = tmp_Img;
                    Train_Img_Members_Mask_Set{i,1}{j,1} = tmp_pickup_Cell_Mask;
                    Train_Img_Members_NucleiMask_Set{i,1}{j,1} = tmp_pickup_Cell_NucleiMask;
                    
                    fprintf('Cell %d in synthetic image %d is OK.\n', j,i);
                    isRatioOK = 1;
                    
                    hit_num_train(end,1) = hit_num_train(end,1) + 1;
                    cell_num_in_hit = cell_num_in_hit + 1;                    
                    fprintf('Hit number %d for cell %d of image %d.\n', hit_num_train(end,1), j, i);
                    hit_num_train(cell_num_in_hit, 1) = 0;
                else                
                    hit_num_train(cell_num_in_hit,1) = hit_num_train(cell_num_in_hit,1) + 1;                    
                end
            end  
        end
    end    
end
    save(strcat(output_path, 'Variables/TrainTransformedCellPool.mat'), ...
        'Train_Img_Members_Set', 'Train_Img_Members_Mask_Set', 'Train_Img_Members_NucleiMask_Set');
    save(strcat(output_path, 'Variables/TrainTransformedCellPool_metaData.mat'), ...
        'hit_num_train');
    
    fprintf('Transform Free-lying Cells (on 512 x 512) for Training Data Set Successfully!\n');
end

%%
%==========================================
% Form Multiply Cells Image & Ground Truth
%                   by
%           TrainDataSet.mat
%==========================================
try
    load(strcat(output_path, 'Train_DataSet.mat'), ...
        'Train_DataSet', 'Train_NucleiMask_DataSet', 'Train_Mask_DataSet');
    fprintf('Read Final Multiply Synthetic Cells Image (on 512 x 512) for Training Data Set Successfully!\n');
catch
    %+---------------------------+
    %| Make Multiply Cells Image |
    %|           for             |
    %|    Training Data Set      |
    %+---------------------------+
    Train_DataSet = cell(num_Train_img,1);
    Train_Mask_DataSet = Train_Img_Members_Mask_Set;
    Train_NucleiMask_DataSet = cell(num_Train_img,1);
    
    variance_gaussian_noise = 0.0005;
    
    blurRange = (0.7:0.01:1)';
    idx_rand_blur_factor = randi(size(blurRange,1));
    blur_factor_gaussian = blurRange(idx_rand_blur_factor);
    
    for i = 1:num_Train_img
        %+----------------------------------------+
        %| Add "BackgroundImg" and Gaussian Noise |
        %+----------------------------------------+
        tmp_Img = BackgroundImg;
        tmp_Img = imnoise(tmp_Img, 'gaussian', 0, variance_gaussian_noise);
        tmp_NucleiMask_Img = false(synIm_size, synIm_size);
        
        %+----------------------------------------+
        %| Put All Free-lying Cells on Background |
        %+----------------------------------------+
        for j = 1:size(Train_Mask_DataSet{i,1},1)
            idx_thisCell = find(Train_Img_Members_Mask_Set{i,1}{j,1} == 1);
            tmp_Img(idx_thisCell) = Train_Img_Members_Set{i,1}{j,1}(idx_thisCell);
            
            %+----------------------------------+
            %| Process Nuclei Mask Ground Truth |
            %+----------------------------------+
            idx_nucleiMask_thisCell = find(Train_Img_Members_NucleiMask_Set{i,1}{j,1} == 1);
            tmp_NucleiMask_Img(idx_nucleiMask_thisCell) = Train_Img_Members_NucleiMask_Set{i,1}{j,1}(idx_nucleiMask_thisCell);
        end
        
        %+----------------------------+
        %| Process Overlapping Region |
        %+----------------------------+
        alphaRange = [0.88:0.01:0.99]';
        idx_rand_alpha_factor = randi(size(alphaRange,1));
        alpha_transition_smooth = alphaRange(idx_rand_alpha_factor);
        for j = 1:size(Train_Mask_DataSet{i,1},1)
            idx_mask_j = find(Train_Mask_DataSet{i,1}{j,1} == 1);
            for k = 1:size(Train_Mask_DataSet{i,1},1)
                if k == j
                    continue;
                end
                idx_mask_k = find(Train_Mask_DataSet{i,1}{k,1} == 1);
                
                idx_shared_region = intersect(idx_mask_j, idx_mask_k);
                if ~isempty(idx_shared_region)
                    tmp_Img(idx_shared_region) = ...
                            max(Train_Img_Members_Set{i,1}{j,1}(idx_shared_region) ...
                            - Train_Img_Members_Set{i,1}{k,1}(idx_shared_region), ...
                            alpha_transition_smooth * min(Train_Img_Members_Set{i,1}{j,1}(idx_shared_region), ...
                            Train_Img_Members_Set{i,1}{k,1}(idx_shared_region)));
                else
                    continue;
                end
            end
        end
        
        %+-----------------------------+
        %| Gaussian blur for the image |
        %+-----------------------------+
        G_blur = fspecial('gaussian',[3 3], blur_factor_gaussian);
        tmp_Img = imfilter(tmp_Img,G_blur,'same');
        
        Train_DataSet{i,1} = tmp_Img;
        Train_NucleiMask_DataSet{i,1} = tmp_NucleiMask_Img;
    end
    
    save(strcat(output_path, 'Train_DataSet.mat'), ...
        'Train_DataSet', 'Train_NucleiMask_DataSet', 'Train_Mask_DataSet');
    
    fprintf('Final Multiply Synthetic Cells Image (on 512 x 512) for Training Data Set Successfully Generated!\n');
end
end